/*
 * Radiometrix RPC driver kernel module header file
 *
 * Jeremy Elson, USC/ISI
 *
 * $Id: krpc.h,v 1.12 2000/07/11 07:53:58 jelson Exp $
 */
/****************** Public Interface ***************************************/

#define RPC_MAJOR_NUMBER_DEFAULT  240

/*
 *     Minor number 0: Raw RPC interface (just like the current driver)
 *     Minor number 1:  Fragmentation interface so that packets larger
 *                     than 27 bytes can be transmitted and/or
 *                     received.
 *     Minor number 2: Packets are delivered with the "rpc_augmented"
 *                     structure prepended (i.e., before the data).
 *                     This structure contains data such as the time of
 *                     reception of the  packet.
 *     Minor number 3: Both 1 and 2
 */
#define RPC_MODE_COOKED     1
#define RPC_MODE_AUGMENTED  2
#define RPC_MODE_NETIF      4

#ifdef __KERNEL__
#include <linux/time.h>
#else
#include <sys/time.h>
#endif

/* the "augmented data" structure */
struct rpc_augmented {
  struct timeval packet_time;
  int spare1;  /* spares can be renamed without requiring userspace recompile*/
  int spare2;
  int spare3;
};


/****************** End of Public Interface ********************************/


/* debugging support */
void rdebug_real(char *fmt, ...)
  __attribute__ ((format (printf, 1, 2)));

#define RDEBUG(message_level, args...) do { \
   if (krpc_debug_level >= message_level) rdebug_real(##args); \
} while(0)

#ifndef __RPC_KM_C__
extern int krpc_debug_level;
#endif


#ifdef __RPC_KM_C__ /* defines only needed in rpc-km.c and friends */

#define RPC_POLL_INTERVAL 1
#define RPC_BUFFER_SIZE 32

/* possible reasons why we are running rpc_run_radio */
#define RPC_TIMER     1
#define RPC_INTERRUPT 2
#define RPC_WRITE     3

#define run_radio(reason) { \
  rpc_queue.data = (void *) reason; \
  rpc_queue.routine = (void *) (void *) rpc_run_radio;\
  queue_task(&rpc_queue, &tq_immediate); \
  mark_bh(IMMEDIATE_BH); }

#define UNKNOWN_TIME(t) (t.tv_sec == unknown_time.tv_sec &&  \
                         t.tv_usec == unknown_time.tv_usec)

static void rpc_run_radio(void *data);
void send_fragment_upcall(caddr_t message, int legnth);
void enqueue_packet(caddr_t message, int length, struct timeval packet_time);


#endif /* __RPC_KM_C__ */
