/*
 * This defines headers for low-level functions used to control the
 * Radiometrix RPC at the pin level.
 *
 * Note that this is a "private API" -- it should NOT be used by
 * applications.  It is really only meant for librpc (the user-land
 * library) and krpc (the kernel library).
 *
 * Applications SHOULD NOT use this API.
 *
 * J. Elson 2 March 2000
 *
 * $Id: rpc_lowlevel.h,v 1.6 2000/07/11 07:53:58 jelson Exp $
 */

/* Constants used by the library */
#define RPC_TIMEOUT -1

/* Initializes RPC pins but doesn't try to send anything.  Should only
 * be called once when the app starts.  Always succeeds. */
void perform_init_rpc(void);

/* Reset the RPC.  Returns 0 on success, or RPC_TIMEOUT on timeout. */
int perform_reset_rpc(void);


/* Poll the radiometrix to determine if a message is waiting.  If so,
 * get it, and call the user-supplied callback if any.  Return value
 * of 1 indicates that a message was waiting and the upcall has been
 * called; 0 means no message was waiting.  RPC_TIMEOUT means an error
 * occurred (usually a timeout) and the RPC should be reset.  */
int perform_receive_message(void);

/* any function linked with this library must provide this function;
 * it is called to process incoming messages when
 * perform_receive_message() is called. */
void rpc_incoming_upcall(caddr_t buf, int length);

/* send a message out to the rpc.  returns the number of bytes written
 * if succesful, or RPC_TIMEOUT if there's an error (including a
 * timeout) */
int perform_send_message(caddr_t message, int length);


/* these functions help split sending into two stages: first, telling
 * the RPC that we have something to send (by clearing TXR); then,
 * polling later to see if the RPC is ready for us to upload the
 * packet. */
void perform_tx_request(void);

/* is the RPC ready to receive?  1 if ready, 0 if not.  valid only if
 * perform_tx_request was previously called.  to maintain sanity,
 * perform_send_message must follow perform_tx_poll if it returns 1!
 */
int perform_tx_poll(void);


/* Sends control bytes to the RPC.  Returns 0 on success or
 * RPC_TIMEOUT on failure such as timeout. */
int perform_send_control(char address, char value);


/*************** Internal Functions (NOT exposed in API) *****************/

int receive_nibble(void);
int receive_byte(void);

int send_nibble(char data);
int send_byte(char data);

#ifdef __KERNEL__
#define RPC_TIMEOUT_INTERVAL 10000
#else
#define RPC_TIMEOUT_INTERVAL 10000
#endif

#define RPC_RETRY_INTERVAL (HZ * 2)
#define RPC_TIMEOUT_TIME (HZ/2)
